package com.steria.digitalsignature.core;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlDate;
import org.apache.xmlbeans.XmlInteger;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;
import org.w3.x2000.x09.xmldsig.X509DataType;

import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.SignatureObjectType;
import x0CoreSchema.oasisNamesTcDss1.SignatureObjectType.Other;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse;
import x0ProfilesXSS.oasisNamesTcDss1.ReturnX509CertificateInfoDocument.ReturnX509CertificateInfo;
import x0ProfilesXSS.oasisNamesTcDss1.X509CertificateInfoDocument.X509CertificateInfo;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.exception.SignaturaInvalidAttributExcepcio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.properties.Messages;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

/**
 * @author cvila
 * Aquestes classes hereten de les pare que les aporten tota la infraestructura necessria per treballar i per tant la classe ValidacioCertificat es centrar en la lgica de negoci prpia per realitzar la petici de validaci dun certificat digital X509
 */
public class ValidacioCertificat extends Validacio implements ConstantsApplicacio
{
    private static final Log log = LogFactory.getLog(ValidacioCertificat.class);
    
    public ValidacioCertificat() {
        super();
    }
    
    /**
     * Aquest mtode crea un Vo per defecte que obt informaci de prova com un certificat digital de prova i altres parmetres del fitxer de propietats. En un cas real aquest mtode no sutilitza i es carreguen dades reals del certificats a traves dels mtodes i atributs del DigitalSignatureContentVo.
     * @throws SignaturaInvalidAttributExcepcio
     * @throws IOException
     */
    public DigitalSignatureContentVo createDefaultVO() throws SignaturaInvalidAttributExcepcio, IOException {
        log.info("createDefaultVO() init"); 
        
        DigitalSignatureContentVo ret = new DigitalSignatureContentVo();
        String certFile = propFile.getProperty(ConfigProperties.PROP_CERT_FILE);
        log.info("default certFile=["+certFile+"]");  
        ret.setCertificate(Utils.getBytesFromFile(certFile));
        // posem el perfil ests (XSS)
        ret.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS);
        log.info("createDefaultVO() done");         
        return ret;
    }
    
    /* 
     * Obt la resposta del servei i la parsetja o tracta afegint-lo al hash datributs resultants del nostre Vo. Un hashmap datributs on la key ser el nom dels atributs que ens interessen en el procs i el valor el contingut del atribut retornat. Per exemple: DigitalSignatureContentVo.atributs.get(ApplicationConfig.NIF) retornar el valor del NIF obtingut a la validaci. Els atributs obligatoris sempre vindran complimentats o amb valor buit i els atributs opcionals safegiran posteriorment.
     */
    protected HashMap<String, String> generaHashResultat(VerifyResponse response) {
        HashMap<String, String> hashAtributs = new HashMap<String, String>();
        
        if (response.getOptionalOutputs()!=null &&
                response.getOptionalOutputs().getX509CertificateInfoArray().length>0) {
            X509CertificateInfo x509CertificateInfo = response.getOptionalOutputs().getX509CertificateInfoArray()[0];
            for (int i=0; i<x509CertificateInfo.sizeOfAttributeArray(); i++) {
                XmlObject[] values = x509CertificateInfo.getAttributeArray(i).getAttributeValueArray();
                if (values.length > 0) {
                    XmlObject xmlObject = values[0];
                    String keyName = x509CertificateInfo.getAttributeArray(i).getName();
                    log.info("xmlObject: " + xmlObject); 
                    if (xmlObject!=null) {
                        if(xmlObject instanceof XmlDate) {
                            XmlDate xmlDate = (XmlDate) xmlObject;
                            hashAtributs.put(keyName,xmlDate.getStringValue());
                            log.info("Date: " + xmlDate.getDateValue());                     
                        }
                        else if(xmlObject instanceof XmlString) {
                            XmlString xmlString = (XmlString) xmlObject;
                            hashAtributs.put(keyName,xmlString.getStringValue());
                            log.info("String: " + xmlString.getStringValue()); 
                        }
                        else if(xmlObject instanceof XmlInteger) {
                            XmlInteger xmlInteger = (XmlInteger) xmlObject;
                            hashAtributs.put(keyName,xmlInteger.getStringValue());
                            log.info("Integer: " + xmlInteger.getStringValue()); 
                        }
                    }
                }
            }
        }
        
        return hashAtributs;
    }
    
    /* 
     * Completar el missatge DSS de petici de signatura verifyRequest i el DigitalSignatureContentVo a travs de la implementaci daquest mtode especfic. En aquest sagafa la informaci del Vo com el certificat i atributs addicionals i s crea la estructura oportuna del missatge DSS de petici de signatura segons la descripci feta daquesta en lapartat de missatges DSS: 
     */
    protected void completarVerifyRequest (DigitalSignatureContentVo vo)
    {
        if (vo instanceof DigitalSignatureContentVo) {
        	DigitalSignatureContentVo certificatVo = (DigitalSignatureContentVo)vo;
            
            try
            {
                vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS);
            }
            catch (SignaturaInvalidAttributExcepcio e)
            {
            }

            // Generamos el tag SignatureObject
            SignatureObjectType signatureObject = this.verifyRequest.addNewSignatureObject();
            // Generamos el tag Other       
            Other otherCertificado = signatureObject.addNewOther();
            // Generamos el tag X509Data        
            X509DataType x509DataType = otherCertificado.addNewX509Data();
            // Generamos el tag X509Certificate
            x509DataType.setX509CertificateArray( new byte[][] { certificatVo.getCertificate() } );
            // Generamos el tag OptionalInputs y dems atributos, si se piden
            OptionalInputs optionalInputs = this.verifyRequest.addNewOptionalInputs();
            optionalInputs.addNewReturnProcessingDetails();
            if (certificatVo.getAttributsOpcionals().size()>0) {
                ReturnX509CertificateInfo returnX509CertificateInfo = optionalInputs.addNewReturnX509CertificateInfo();
                for (String atribut : certificatVo.getAttributsOpcionals()) {
                    returnX509CertificateInfo.addNewAttributeDesignator().setName(atribut);
                }        
                optionalInputs.setReturnX509CertificateInfoArray( new ReturnX509CertificateInfo[] { returnX509CertificateInfo } );
            }
        }
        else {
            throw new IllegalArgumentException(Messages.getString("ValidacioCertificat.0")); 
        }
    }
}
