package com.steria.digitalsignature.vo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.exception.SignaturaInvalidAttributExcepcio;

/**
 * @author cvila
 * Aquesta classe, DigitalSignatureContentVo, contindr tots els atributs dentrada i sortida necessria pels mtodes de PKI i per la invocaci dels serveis de PSIS, de manera que amb aquesta classe de transport de dades tindrem totes les dades importants i significatives dels nostres processos de signatura.
 */
public class DigitalSignatureContentVo implements Serializable, ConstantsApplicacio
{

    private static final long serialVersionUID = -8757957805503697597L;
    
    protected byte[] data;
    protected byte[] certificate;
    protected byte[] signature;
    protected String inlineXML;
    
    protected List<String> attributsOpcionals = new ArrayList<String> ();
    protected List<String> operacionsAddicionals = new ArrayList<String> ();
    protected String perfil;
    protected HashMap<?, ?> atributs;     
    protected TYPE type;
    
    // Return properties
    protected HashMap<String, String> msg;  // retorna els missatges informatius generats en el proces de validacio o operacio donada.
    protected boolean procesOk;
    
    public DigitalSignatureContentVo()
    {
    }
    
    /**
     * @param data - El contingut del Certificat de lusuari (X509Certificate.getEncoded())  o b en el cas duna sollicitud de segell de temps contindr el contingut del Certificat de la TSA que volem que ens emeti el segell (el segell tornar signat per la seva clau privada).
     */
    public DigitalSignatureContentVo(byte[] data)
    {
        this.data = data;
    }
    
    /**
     * @param inlineXML
     * @param attributsOpcionals - un array amb els atributs opcionals a normalitzar i obtenir que es guardaran com a constants a la classes del component de constants daplicaci ApplicationConfig
     * @param operacionsAddicionals - un array amb altres atributs de configuraci com ara el perfil DSS
     * @param perfil
     * @param atributs
     * @param type - indica el tipus de dades que s'envien al stream de bytes (certificat, signatura, document, etc.)
     */
    public DigitalSignatureContentVo(String inlineXML, List<String> attributsOpcionals, List<String> operacionsAddicionals, String perfil, HashMap<?, ?> atributs, TYPE type)
    {
        this.inlineXML = inlineXML;
        this.attributsOpcionals = attributsOpcionals;
        this.operacionsAddicionals = operacionsAddicionals;
        this.perfil = perfil;
        this.atributs = atributs;
        this.type = type;
    }
    
    /**
     * @return Un hashmap datributs on la key ser el nom dels atributs que ens interessen en el procs i el valor el contingut del atribut retornat. Per exemple: DigitalSignatureContentVo.atributs.get(ApplicationConfig.NIF) retornar el valor del NIF obtingut a la validaci. Els atributs obligatoris sempre vindran complimentats o amb valor buit i els atributs opcionals safegiran posteriorment.
     */
    public HashMap<?, ?> getAtributs()
    {
        return atributs;
    }
    /**
     * @param atributs Un hashmap datributs on la key ser el nom dels atributs que ens interessen en el procs i el valor el contingut del atribut retornat. Per exemple: DigitalSignatureContentVo.atributs.get(ApplicationConfig.NIF) retornar el valor del NIF obtingut a la validaci. Els atributs obligatoris sempre vindran complimentats o amb valor buit i els atributs opcionals safegiran posteriorment.
     */
    public void setAtributs(HashMap<?, ?> atributs)
    {
        this.atributs = atributs;
    }
    public List<String> getAttributsOpcionals()
    {
        return attributsOpcionals;
    }
    /**
     * @param attributsOpcionals Una llista amb els atributs opcionals a normalitzar i obtenir que es guardaran com a constants a la classes del component de constants daplicaci ConstantsApplicacio. Els atributs obligatoris els passar automticament el component. Poden  anar apareixent nous atributs al estndard DSS o els seus profiles que ens puguin interessar o noves necessitats que es detectin, noms caldr afegir-ho a la nostra classe de constants ConstantsApplicacio
     * @throws SignaturaInvalidAttributExcepcio
     */
    public void setAttributsOpcionals(List<String> attributsOpcionals) throws SignaturaInvalidAttributExcepcio
    {
        this.attributsOpcionals = attributsOpcionals;
    }
    public String getInlineXML()
    {
        return inlineXML;
    }
    /**
     * @param inlineXML - El contingut duna signatura XML detached (InlineXML)
     */
    public void setInlineXML(String inlineXML)
    {
        this.inlineXML = inlineXML;
    }
    public List<String> getOperacionsAddicionals()
    {
        return operacionsAddicionals;
    }
    /**
     * @param operacionsAddicionals - Una llista amb les operacions addicionals possibles dintre del servei concret de signatura o PKI. Aquestes operacions addicionals i opcionals quedaran registrades tamb a la classe ConstantsApplicacio
     * @throws SignaturaInvalidAttributExcepcio
     */
    public void setOperacionsAddicionals(List<String> operacionsAddicionals) throws SignaturaInvalidAttributExcepcio
    {
        this.operacionsAddicionals = operacionsAddicionals;
    }
    public String getPerfil()
    {
        return perfil;
    }
    /**
     * @param perfil - Un atribut de protocol on sindicar el tipus de perfil DSS de treball. Per exemple si treballem amb lextensi de CATCert XSS, amb el perfil especfic de serveis per PDF o be el de segell de temps
     * @throws SignaturaInvalidAttributExcepcio
     */
    public void setPerfil(String perfil) throws SignaturaInvalidAttributExcepcio
    {
        this.perfil = perfil;
    }
    public TYPE getType()
    {
        return type;
    }
    public void setType(TYPE type)
    {
        this.type = type;
    }
    public HashMap<String, String> getMsg()
    {
        return msg;
    }
    /**
     * @param msg - Un hashmap on afegirem tota la informaci de validaci i detalls de sortida del procs
     */
    public void setMsg(HashMap<String, String> msg)
    {
        this.msg = msg;
    }
    public boolean isProcesOk()
    {
        return procesOk;
    }
    /**
     * @param procesOk - Un camp de resultat boole que indicar si el procs s correcte (true  sha validat positivament la signatura) o ha fallat (false  no sha validat la signatura).  Pot ser correcte si es valida el certificat, si es valida la signatura, si es retorna segell de temps o pot se fals si dona qualsevol excepci (no hi ha connexi, etc.) o no es valida el certificat, no es valida la signatura, etc
     * La variable booleana procesOk on espera que el resultat del missatge dss:ResultMajor sigui el resultat definit com a correcte urn\:oasis\:names \:tc\:dss\:1.0\:resultmajor\:Success (configurable al properties).
     */
    public void setProcesOk(boolean procesOk)
    {
        this.procesOk = procesOk;
    }
    public byte[] getData()
    {
        return data;
    }
    /**
     * @param data- El resum del document a signar (o posar segell de temps) (DocumentHash)
     * El contingut del document pdf amb signatura CMS (Base64Data)
     */
    public void setData(byte[] data)
    {
        this.data = data;
    }
    public byte[] getCertificate()
    {
        return certificate;
    }
    /**
     * @param certificate - El contingut del Certificat  X509 de lusuari en base64 (X509Certificate) o b en el cas duna sollicitud de segell de temps contindr el contingut del Certificat de la TSA que volem que ens emeti el segell.
     */
    public void setCertificate(byte[] certificate)
    {
        this.certificate = certificate;
    }
    public byte[] getSignature()
    {
        return signature;
    }
    /**
     * @param signature - El contingut de la signatura de tipus CMS (Base64Signature)
     */
    public void setSignature(byte[] signature)
    {
        this.signature = signature;
    }
}
